// #!/usr/bin/env babel-node
// -*- coding: utf-8 -*-

/** @module helper */
'use strict';
/* !
    region header
    Copyright Torben Sickert (info["~at~"]torben.website) 16.12.2012

    License
    -------

    This library written by Torben Sickert stand under a creative commons
    naming 3.0 unported license.
    See https://creativecommons.org/licenses/by/3.0/deed.de
    endregion
*/
// region imports

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.Helper = exports.KNOWN_FILE_EXTENSIONS = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _clientnode = _interopRequireDefault(require("clientnode"));

var _jsdom = require("jsdom");

var _fs = _interopRequireDefault(require("fs"));

var _path2 = _interopRequireDefault(require("path"));

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

// endregion
// region constants
var KNOWN_FILE_EXTENSIONS = ['js', 'ts', 'json', 'css', 'eot', 'gif', 'html', 'ico', 'jpg', 'png', 'ejs', 'svg', 'ttf', 'woff', '.woff2']; // endregion
// region methods

/**
 * Provides a class of static methods with generic use cases.
 */

exports.KNOWN_FILE_EXTENSIONS = KNOWN_FILE_EXTENSIONS;

var Helper = /*#__PURE__*/function () {
  function Helper() {
    (0, _classCallCheck2["default"])(this, Helper);
  }

  (0, _createClass2["default"])(Helper, null, [{
    key: "isFilePathInLocation",
    // region boolean

    /**
     * Determines whether given file path is within given list of file
     * locations.
     * @param filePath - Path to file to check.
     * @param locationsToCheck - Locations to take into account.
     * @returns Value "true" if given file path is within one of given
     * locations or "false" otherwise.
     */
    value: function isFilePathInLocation(filePath, locationsToCheck) {
      var _iterator = _createForOfIteratorHelper(locationsToCheck),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var pathToCheck = _step.value;
          if (_path2["default"].resolve(filePath).startsWith(_path2["default"].resolve(pathToCheck))) return true;
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return false;
    } // endregion
    // region string
    // TODO test

    /**
     * In-places given assets types in given dom nodes found in given assets.
     * @param assetType - Asset type integration configuration.
     * @param assets - Pool of assets to integrate.
     * @param domNodes - Dom nodes with asset references to include.
     * @param assetPositionPattern - Specifies where current asset content
     * should be placed.
     * @returns Files to remove (caused be integration) and styles to
     * integrate.
     */

  }, {
    key: "inPlaceAssetReferences",
    value: function inPlaceAssetReferences(assetType, assets, domNodes, assetPositionPattern) {
      var filePathsToRemove = [];
      var inPlaceStyleContents = [];

      var _iterator2 = _createForOfIteratorHelper(domNodes),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var domNode = _step2.value;

          var _path = domNode.getAttribute(assetType.attributeName);

          if (typeof _path === 'string') {
            _path = _path.replace(/&.*/g, '');
            if (!Object.prototype.hasOwnProperty.call(assets, _path)) continue;
            var inPlaceDomNode = window.document.createElement(assetType.tagName);

            if (assetType.tagName === 'style') {
              inPlaceDomNode.setAttribute('weboptimizerinplace', 'true');
              inPlaceStyleContents.push(assets[_path].source());
            } else inPlaceDomNode.textContent = assets[_path].source();

            if (domNode.parentNode) {
              if (assetType.patterns[assetPositionPattern] === 'body') window.document.body.appendChild(inPlaceDomNode);else if (assetType.patterns[assetPositionPattern] === 'in') domNode.parentNode.insertBefore(inPlaceDomNode, domNode);else if (assetType.patterns[assetPositionPattern] === 'head') window.document.head.appendChild(inPlaceDomNode);else {
                var regularExpressionPattern = '(after|before|in):(.+)';
                var testMatch = new RegExp(regularExpressionPattern).exec(assetType.patterns[assetPositionPattern]);
                var match = void 0;
                if (testMatch) match = testMatch;else throw new Error('Given in place specification "' + "".concat(assetType.patterns[assetPositionPattern]) + "\" for ".concat(assetType.tagName, " does not satisfy") + ' the specified pattern "' + "".concat(regularExpressionPattern, "\"."));

                var _domNode = window.document.querySelector(match[2]);

                if (!_domNode) throw new Error("Specified dom node \"".concat(match[2], "\" could not ") + 'be found to in place "' + "".concat(assetPositionPattern, "\"."));
                if (match[1] === 'in') _domNode.appendChild(inPlaceDomNode);else if (match[1] === 'before') _domNode.insertAdjacentElement('beforebegin', inPlaceDomNode);else _domNode.insertAdjacentElement('afterend', inPlaceDomNode);
              }
              domNode.parentNode.removeChild(domNode);
              /*
                  NOTE: This doesn't prevent webpack from creating this
                  file if present in another chunk so removing it (and a
                  potential source map file) later in the "done" hook.
              */

              filePathsToRemove.push(Helper.stripLoader(_path));
              delete assets[_path];
            } else throw new Error('Given in place specification "' + "".concat(assetType.patterns[assetPositionPattern], "\" for ") + "".concat(assetType.tagName, " is not possible because of ") + 'missing parent node.');
          } else throw new Error('Given in place specification "' + "".concat(assetType.patterns[assetPositionPattern], "\" for ") + "".concat(assetType.tagName, " is not possible because of missing") + " attribute \"".concat(assetType.attributeName, "\"."));
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      return {
        filePathsToRemove: filePathsToRemove,
        inPlaceStyleContents: inPlaceStyleContents
      };
    }
    /**
     * In places each matching cascading style sheet or javaScript file
     * reference.
     * @param content - Markup content to process.
     * @param cascadingStyleSheetPatterns - Pattern to match cascading style
     * sheet asset references again.
     * @param javaScriptPatterns - Pattern to match javaScript asset references
     * again.
     * @param basePath - Base path to use as prefix for file references.
     * @param cascadingStyleSheetChunkNameTemplate - Cascading style sheet
     * chunk name template to use for asset matching.
     * @param javaScriptChunkNameTemplate - JavaScript chunk name template to
     * use for asset matching.
     * @param assets - Mapping of asset file paths to their content.
     * @returns Given an transformed markup.
     */

  }, {
    key: "inPlaceCSSAndJavaScriptAssetReferences",
    value: function inPlaceCSSAndJavaScriptAssetReferences(content, cascadingStyleSheetPatterns, javaScriptPatterns, basePath, cascadingStyleSheetChunkNameTemplate, javaScriptChunkNameTemplate, assets) {
      /*
          NOTE: We have to prevent creating native "style" dom nodes to
          prevent jsdom from parsing the entire cascading style sheet. Which
          is error prune and very resource intensive.
      */
      var styleContents = [];
      content = content.replace(/(<style[^>]*>)([\s\S]*?)(<\/style[^>]*>)/gi, function (match, startTag, content, endTag) {
        styleContents.push(content);
        return "".concat(startTag).concat(endTag);
      });
      /*
          NOTE: We have to translate template delimiter to html compatible
          sequences and translate it back later to avoid unexpected escape
          sequences in resulting html.
      */

      var window = new _jsdom.JSDOM(content.replace(/<%/g, '##+#+#+##').replace(/%>/g, '##-#-#-##')).window;
      var inPlaceStyleContents = [];
      var filePathsToRemove = [];

      for (var _i = 0, _arr = [{
        attributeName: 'href',
        hash: 'hash',
        linkTagName: 'link',
        patterns: cascadingStyleSheetPatterns,
        selector: '[href*=".css"]',
        tagName: 'style',
        template: cascadingStyleSheetChunkNameTemplate
      }, {
        attributeName: 'src',
        hash: 'hash',
        linkTagName: 'script',
        patterns: javaScriptPatterns,
        selector: '[href*=".js"]',
        tagName: 'script',
        template: javaScriptChunkNameTemplate
      }]; _i < _arr.length; _i++) {
        var assetType = _arr[_i];
        if (assetType.patterns) for (var pattern in assetType.patterns) {
          var _Helper$renderFilePat;

          if (!Object.prototype.hasOwnProperty.call(assetType.patterns, pattern)) continue;
          var selector = assetType.selector;
          if (pattern !== '*') selector = "[".concat(assetType.attributeName, "^=\"") + _path2["default"].relative(basePath, Helper.renderFilePathTemplate(assetType.template, (_Helper$renderFilePat = {}, (0, _defineProperty2["default"])(_Helper$renderFilePat, "[".concat(assetType.hash, "]"), ''), (0, _defineProperty2["default"])(_Helper$renderFilePat, '[id]', pattern), (0, _defineProperty2["default"])(_Helper$renderFilePat, '[name]', pattern), _Helper$renderFilePat))) + '"]';
          var domNodes = Array.from(window.document.querySelectorAll("".concat(assetType.linkTagName).concat(selector)));

          if (domNodes.length) {
            var result = Helper.inPlaceAssetReferences(assetType, assets, domNodes, pattern);
            filePathsToRemove = filePathsToRemove.concat(result.filePathsToRemove);
            inPlaceStyleContents = inPlaceStyleContents.concat(result.inPlaceStyleContents);
          } else console.warn("No referenced ".concat(assetType.tagName, " file in ") + 'resulting markup found with selector: "' + "".concat(assetType.linkTagName).concat(assetType.selector, "\""));
        }
      } // NOTE: We have to restore template delimiter and style contents.


      return {
        content: (content.replace(/^(\s*<!doctype [^>]+?>\s*)[\s\S]*$/i, '$1') + window.document.documentElement.outerHTML).replace(/##\+#\+#\+##/g, '<%').replace(/##-#-#-##/g, '%>').replace(/(<style[^>]*>)[\s\S]*?(<\/style[^>]*>)/gi, function (match, startTag, endTag) {
          if (startTag.includes(' weboptimizerinplace="true"')) return startTag.replace(' weboptimizerinplace="true"', '') + "".concat(inPlaceStyleContents.shift()).concat(endTag);
          return "".concat(startTag).concat(styleContents.shift()).concat(endTag);
        }),
        filePathsToRemove: filePathsToRemove
      };
    }
    /**
     * Strips loader informations form given module request including loader
     * prefix and query parameter.
     * @param moduleID - Module request to strip.
     * @returns Given module id stripped.
     */

  }, {
    key: "stripLoader",
    value: function stripLoader(moduleID) {
      moduleID = moduleID.toString();
      var moduleIDWithoutLoader = moduleID.substring(moduleID.lastIndexOf('!') + 1);
      return moduleIDWithoutLoader.includes('?') ? moduleIDWithoutLoader.substring(0, moduleIDWithoutLoader.indexOf('?')) : moduleIDWithoutLoader;
    } // endregion
    // region array

    /**
     * Converts given list of path to a normalized list with unique values.
     * @param paths - File paths.
     * @returns The given file path list with normalized unique values.
     */

  }, {
    key: "normalizePaths",
    value: function normalizePaths(paths) {
      return Array.from(new Set(paths.map(function (givenPath) {
        givenPath = _path2["default"].normalize(givenPath);
        if (givenPath.endsWith('/')) return givenPath.substring(0, givenPath.length - 1);
        return givenPath;
      })));
    } // endregion
    // region file handler

    /**
     * Applies file path/name placeholder replacements with given bundle
     * associated informations.
     * @param template - File path to process placeholder in.
     * @param scope - Scope to use for processing.
     * @returns Processed file path.
     */

  }, {
    key: "renderFilePathTemplate",
    value: function renderFilePathTemplate(template) {
      var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      scope = _clientnode["default"].extend({
        '[chunkhash]': '.__dummy__',
        '[contenthash]': '.__dummy__',
        '[fullhash]': '.__dummy__',
        '[id]': '.__dummy__',
        '[name]': '.__dummy__'
      }, scope);
      var filePath = template;

      for (var placeholderName in scope) {
        if (Object.prototype.hasOwnProperty.call(scope, placeholderName)) filePath = filePath.replace(new RegExp(_clientnode["default"].stringEscapeRegularExpressions(placeholderName), 'g'), "".concat(scope[placeholderName]));
      }

      return filePath;
    }
    /**
     * Converts given request to a resolved request with given context
     * embedded.
     * @param request - Request to determine.
     * @param context - Context of given request to resolve relative to.
     * @param referencePath - Path to resolve local modules relative to.
     * @param aliases - Mapping of aliases to take into account.
     * @param moduleReplacements - Mapping of replacements to take into
     * account.
     * @param relativeModuleLocations - List of relative directory paths to
     * search for modules in.
     * @returns A new resolved request.
     */

  }, {
    key: "applyContext",
    value: function applyContext(request) {
      var context = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : './';
      var referencePath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : './';
      var aliases = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var moduleReplacements = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
      var relativeModuleLocations = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : ['node_modules'];
      referencePath = _path2["default"].resolve(referencePath);

      if (request.startsWith('./') && _path2["default"].resolve(context) !== referencePath) {
        request = _path2["default"].resolve(context, request);

        var _iterator3 = _createForOfIteratorHelper(relativeModuleLocations),
            _step3;

        try {
          for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
            var modulePath = _step3.value;

            var pathPrefix = _path2["default"].resolve(referencePath, modulePath);

            if (request.startsWith(pathPrefix)) {
              request = request.substring(pathPrefix.length);
              if (request.startsWith('/')) request = request.substring(1);
              return Helper.applyModuleReplacements(Helper.applyAliases(request.substring(request.lastIndexOf('!') + 1), aliases), moduleReplacements);
            }
          }
        } catch (err) {
          _iterator3.e(err);
        } finally {
          _iterator3.f();
        }

        if (request.startsWith(referencePath)) {
          request = request.substring(referencePath.length);
          if (request.startsWith('/')) request = request.substring(1);
          return Helper.applyModuleReplacements(Helper.applyAliases(request.substring(request.lastIndexOf('!') + 1), aliases), moduleReplacements);
        }
      }

      return request;
    }
    /**
     * Check if given request points to an external dependency not maintained
     * by current package context.
     * @param request - Request to determine.
     * @param context - Context of current project.
     * @param requestContext - Context of given request to resolve relative to.
     * @param normalizedGivenInjection - Mapping of chunk names to modules
     * which should be injected.
     * @param relativeExternalModuleLocations - Array of paths where external
     * modules take place.
     * @param aliases - Mapping of aliases to take into account.
     * @param moduleReplacements - Mapping of replacements to take into
     * account.
     * @param extensions - List of file and module extensions to take into
     * account.
     * @param referencePath - Path to resolve local modules relative to.
     * @param pathsToIgnore - Paths which marks location to ignore.
     * @param relativeModuleLocations - List of relative file path to search
     * for modules in.
     * @param packageEntryFileNames - List of package entry file names to
     * search for. The magic name "__package__" will search for an appreciate
     * entry in a "package.json" file.
     * @param packageMainPropertyNames - List of package file main property
     * names to search for package representing entry module definitions.
     * @param packageAliasPropertyNames - List of package file alias property
     * names to search for package specific module aliases.
     * @param includePattern - Array of regular expressions to explicitly mark
     * as external dependency.
     * @param excludePattern - Array of regular expressions to explicitly mark
     * as internal dependency.
     * @param inPlaceNormalLibrary - Indicates whether normal libraries should
     * be external or not.
     * @param inPlaceDynamicLibrary - Indicates whether requests with
     * integrated loader configurations should be marked as external or not.
     * @param encoding - Encoding for file names to use during file traversing.
     * @returns A new resolved request indicating whether given request is an
     * external one.
     */

  }, {
    key: "determineExternalRequest",
    value: function determineExternalRequest(request) {
      var context = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : './';
      var requestContext = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : './';
      var normalizedGivenInjection = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var relativeExternalModuleLocations = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : ['node_modules'];
      var aliases = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};
      var moduleReplacements = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};
      var extensions = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : {
        file: {
          external: ['.compiled.js', '.js', '.json'],
          internal: KNOWN_FILE_EXTENSIONS.map(function (suffix) {
            return ".".concat(suffix);
          })
        }
      };
      var referencePath = arguments.length > 8 && arguments[8] !== undefined ? arguments[8] : './';
      var pathsToIgnore = arguments.length > 9 && arguments[9] !== undefined ? arguments[9] : ['.git'];
      var relativeModuleLocations = arguments.length > 10 && arguments[10] !== undefined ? arguments[10] : ['node_modules'];
      var packageEntryFileNames = arguments.length > 11 && arguments[11] !== undefined ? arguments[11] : ['index', 'main'];
      var packageMainPropertyNames = arguments.length > 12 && arguments[12] !== undefined ? arguments[12] : ['main', 'module'];
      var packageAliasPropertyNames = arguments.length > 13 && arguments[13] !== undefined ? arguments[13] : [];
      var includePattern = arguments.length > 14 && arguments[14] !== undefined ? arguments[14] : [];
      var excludePattern = arguments.length > 15 && arguments[15] !== undefined ? arguments[15] : [];
      var inPlaceNormalLibrary = arguments.length > 16 && arguments[16] !== undefined ? arguments[16] : false;
      var inPlaceDynamicLibrary = arguments.length > 17 && arguments[17] !== undefined ? arguments[17] : true;
      var encoding = arguments.length > 18 && arguments[18] !== undefined ? arguments[18] : 'utf-8';
      context = _path2["default"].resolve(context);
      requestContext = _path2["default"].resolve(requestContext);
      referencePath = _path2["default"].resolve(referencePath); // NOTE: We apply alias on externals additionally.

      var resolvedRequest = Helper.applyModuleReplacements(Helper.applyAliases(request.substring(request.lastIndexOf('!') + 1), aliases), moduleReplacements);
      if (_clientnode["default"].isAnyMatching(resolvedRequest, excludePattern)) return null;
      /*
          NOTE: Aliases and module replacements doesn't have to be forwarded
          since we pass an already resolved request.
      */

      var filePath = Helper.determineModuleFilePath(resolvedRequest, {}, {}, {
        file: extensions.file.external
      }, context, requestContext, pathsToIgnore, relativeModuleLocations, packageEntryFileNames, packageMainPropertyNames, packageAliasPropertyNames, encoding);
      /*
          NOTE: We mark dependencies as external if there file couldn't be
          resolved or are specified to be external explicitly.
      */

      if (!(filePath || inPlaceNormalLibrary) || _clientnode["default"].isAnyMatching(resolvedRequest, includePattern)) return Helper.applyContext(resolvedRequest, requestContext, referencePath, aliases, moduleReplacements, relativeModuleLocations);

      for (var chunkName in normalizedGivenInjection) {
        if (Object.prototype.hasOwnProperty.call(normalizedGivenInjection, chunkName)) {
          var _iterator4 = _createForOfIteratorHelper(normalizedGivenInjection[chunkName]),
              _step4;

          try {
            for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
              var moduleID = _step4.value;
              if (Helper.determineModuleFilePath(moduleID, aliases, moduleReplacements, {
                file: extensions.file.internal
              }, context, requestContext, pathsToIgnore, relativeModuleLocations, packageEntryFileNames, packageMainPropertyNames, packageAliasPropertyNames, encoding) === filePath) return null;
            }
          } catch (err) {
            _iterator4.e(err);
          } finally {
            _iterator4.f();
          }
        }
      }

      var parts = context.split('/');
      var externalModuleLocations = [];

      while (parts.length > 0) {
        var _iterator5 = _createForOfIteratorHelper(relativeExternalModuleLocations),
            _step5;

        try {
          for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
            var relativePath = _step5.value;
            externalModuleLocations.push(_path2["default"].join('/', parts.join('/'), relativePath));
          }
        } catch (err) {
          _iterator5.e(err);
        } finally {
          _iterator5.f();
        }

        parts.splice(-1, 1);
      }
      /*
          NOTE: We mark dependencies as external if they does not contain a
          loader in their request and aren't part of the current main package
          or have a file extension other than javaScript aware.
      */


      if (!inPlaceNormalLibrary && (extensions.file.external.length === 0 || filePath && extensions.file.external.includes(_path2["default"].extname(filePath)) || !filePath && extensions.file.external.includes('')) && !(inPlaceDynamicLibrary && request.includes('!')) && (!filePath && inPlaceDynamicLibrary || filePath && (!filePath.startsWith(context) || Helper.isFilePathInLocation(filePath, externalModuleLocations)))) return Helper.applyContext(resolvedRequest, requestContext, referencePath, aliases, moduleReplacements, relativeModuleLocations);
      return null;
    }
    /**
     * Determines asset type of given file.
     * @param filePath - Path to file to analyse.
     * @param buildConfiguration - Meta informations for available asset
     * types.
     * @param paths - List of paths to search if given path doesn't reference
     * a file directly.
     * @returns Determined file type or "null" of given file couldn't be
     * determined.
     */

  }, {
    key: "determineAssetType",
    value: function determineAssetType(filePath, buildConfiguration, paths) {
      var result = null;

      for (var type in buildConfiguration) {
        if (_path2["default"].extname(filePath) === ".".concat(buildConfiguration[type].extension)) {
          result = type;
          break;
        }
      }

      if (!result) for (var _i2 = 0, _arr2 = [paths.source, paths.target]; _i2 < _arr2.length; _i2++) {
        var _type = _arr2[_i2];

        for (var assetType in _type.asset) {
          if (Object.prototype.hasOwnProperty.call(_type.asset, assetType) && assetType !== 'base' && _type.asset[assetType] && filePath.startsWith(_type.asset[assetType])) return assetType;
        }
      }
      return result;
    }
    /**
     * Adds a property with a stored array of all matching file paths, which
     * matches each build configuration in given entry path and converts given
     * build configuration into a sorted array were javaScript files takes
     * precedence.
     * @param configuration - Given build configurations.
     * @param entryPath - Path to analyse nested structure.
     * @param pathsToIgnore - Paths which marks location to ignore.
     * @param mainFileBasenames - File basenames to sort into the front.
     * @returns Converted build configuration.
     */

  }, {
    key: "resolveBuildConfigurationFilePaths",
    value: function resolveBuildConfigurationFilePaths(configuration) {
      var entryPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : './';
      var pathsToIgnore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : ['.git'];
      var mainFileBasenames = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : ['index', 'main'];
      var buildConfiguration = [];

      for (var type in configuration) {
        if (Object.prototype.hasOwnProperty.call(configuration, type)) {
          var newItem = _clientnode["default"].extend(true, {
            filePaths: []
          }, configuration[type]);

          var _iterator6 = _createForOfIteratorHelper(_clientnode["default"].walkDirectoryRecursivelySync(entryPath, function (file) {
            if (Helper.isFilePathInLocation(file.path, pathsToIgnore)) return false;
          })),
              _step6;

          try {
            for (_iterator6.s(); !(_step6 = _iterator6.n()).done;) {
              var _file$stats;

              var file = _step6.value;
              if ((_file$stats = file.stats) !== null && _file$stats !== void 0 && _file$stats.isFile() && _path2["default"].extname(file.path).endsWith(newItem.extension) && !new RegExp(newItem.filePathPattern).test(file.path)) newItem.filePaths.push(file.path);
            }
          } catch (err) {
            _iterator6.e(err);
          } finally {
            _iterator6.f();
          }

          newItem.filePaths.sort(function (firstFilePath, secondFilePath) {
            if (mainFileBasenames.includes(_path2["default"].basename(firstFilePath, _path2["default"].extname(firstFilePath)))) {
              if (mainFileBasenames.includes(_path2["default"].basename(secondFilePath, _path2["default"].extname(secondFilePath)))) return 0;
            } else if (mainFileBasenames.includes(_path2["default"].basename(secondFilePath, _path2["default"].extname(secondFilePath)))) return 1;

            return 0;
          });
          buildConfiguration.push(newItem);
        }
      }

      return buildConfiguration.sort(function (first, second) {
        if (first.outputExtension !== second.outputExtension) {
          if (first.outputExtension === 'js') return -1;
          if (second.outputExtension === 'js') return 1;
          return first.outputExtension < second.outputExtension ? -1 : 1;
        }

        return 0;
      });
    }
    /**
     * Determines all file and directory paths related to given internal
     * modules as array.
     * @param givenInjection - List of module ids or module file paths.
     * @param aliases - Mapping of aliases to take into account.
     * @param moduleReplacements - Mapping of module replacements to take into
     * account.
     * @param extensions - List of file and module extensions to take into
     * account.
     * @param context - File path to resolve relative to.
     * @param referencePath - Path to search for local modules.
     * @param pathsToIgnore - Paths which marks location to ignore.
     * @param relativeModuleLocations - List of relative file path to search
     * for modules in.
     * @param packageEntryFileNames - List of package entry file names to
     * search for. The magic name "__package__" will search for an appreciate
     * entry in a "package.json" file.
     * @param packageMainPropertyNames - List of package file main property
     * names to search for package representing entry module definitions.
     * @param packageAliasPropertyNames - List of package file alias property
     * names to search for package specific module aliases.
     * @param encoding - File name encoding to use during file traversing.
     * @returns Object with a file path and directory path key mapping to
     * corresponding list of paths.
     */

  }, {
    key: "determineModuleLocations",
    value: function determineModuleLocations(givenInjection) {
      var aliases = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var moduleReplacements = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var extensions = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
        file: KNOWN_FILE_EXTENSIONS.map(function (suffix) {
          return ".".concat(suffix);
        })
      };
      var context = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : './';
      var referencePath = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : '';
      var pathsToIgnore = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : ['.git'];
      var relativeModuleLocations = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : ['node_modules'];
      var packageEntryFileNames = arguments.length > 8 && arguments[8] !== undefined ? arguments[8] : ['__package__', '', 'index', 'main'];
      var packageMainPropertyNames = arguments.length > 9 && arguments[9] !== undefined ? arguments[9] : ['main', 'module'];
      var packageAliasPropertyNames = arguments.length > 10 && arguments[10] !== undefined ? arguments[10] : [];
      var encoding = arguments.length > 11 && arguments[11] !== undefined ? arguments[11] : 'utf-8';
      var filePaths = [];
      var directoryPaths = [];
      var normalizedGivenInjection = Helper.resolveModulesInFolders(Helper.normalizeGivenInjection(givenInjection), aliases, moduleReplacements, context, referencePath, pathsToIgnore);

      for (var chunkName in normalizedGivenInjection) {
        if (Object.prototype.hasOwnProperty.call(normalizedGivenInjection, chunkName)) {
          var _iterator7 = _createForOfIteratorHelper(normalizedGivenInjection[chunkName]),
              _step7;

          try {
            for (_iterator7.s(); !(_step7 = _iterator7.n()).done;) {
              var moduleID = _step7.value;
              var filePath = Helper.determineModuleFilePath(moduleID, aliases, moduleReplacements, extensions, context, referencePath, pathsToIgnore, relativeModuleLocations, packageEntryFileNames, packageMainPropertyNames, packageAliasPropertyNames, encoding);

              if (filePath) {
                filePaths.push(filePath);

                var directoryPath = _path2["default"].dirname(filePath);

                if (!directoryPaths.includes(directoryPath)) directoryPaths.push(directoryPath);
              }
            }
          } catch (err) {
            _iterator7.e(err);
          } finally {
            _iterator7.f();
          }
        }
      }

      return {
        filePaths: filePaths,
        directoryPaths: directoryPaths
      };
    }
    /**
     * Determines a list of concrete file paths for given module id pointing to
     * a folder which isn't a package.
     * @param normalizedGivenInjection - Injection data structure of modules
     * with folder references to resolve.
     * @param aliases - Mapping of aliases to take into account.
     * @param moduleReplacements - Mapping of replacements to take into
     * account.
     * @param context - File path to determine relative to.
     * @param referencePath - Path to resolve local modules relative to.
     * @param pathsToIgnore - Paths which marks location to ignore.
     * @returns Given injections with resolved folder pointing modules.
     */

  }, {
    key: "resolveModulesInFolders",
    value: function resolveModulesInFolders(normalizedGivenInjection) {
      var aliases = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var moduleReplacements = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var context = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : './';
      var referencePath = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : '';
      var pathsToIgnore = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : ['.git'];
      if (referencePath.startsWith('/')) referencePath = _path2["default"].relative(context, referencePath);

      for (var chunkName in normalizedGivenInjection) {
        if (Object.prototype.hasOwnProperty.call(normalizedGivenInjection, chunkName)) {
          var index = 0;

          var _iterator8 = _createForOfIteratorHelper(normalizedGivenInjection[chunkName]),
              _step8;

          try {
            for (_iterator8.s(); !(_step8 = _iterator8.n()).done;) {
              var moduleID = _step8.value;
              moduleID = Helper.applyModuleReplacements(Helper.applyAliases(Helper.stripLoader(moduleID), aliases), moduleReplacements);

              var resolvedPath = _path2["default"].resolve(referencePath, moduleID);

              if (_clientnode["default"].isDirectorySync(resolvedPath)) {
                normalizedGivenInjection[chunkName].splice(index, 1);

                var _iterator9 = _createForOfIteratorHelper(_clientnode["default"].walkDirectoryRecursivelySync(resolvedPath, function (file) {
                  if (Helper.isFilePathInLocation(file.path, pathsToIgnore)) return false;
                })),
                    _step9;

                try {
                  for (_iterator9.s(); !(_step9 = _iterator9.n()).done;) {
                    var _file$stats2;

                    var file = _step9.value;
                    if ((_file$stats2 = file.stats) !== null && _file$stats2 !== void 0 && _file$stats2.isFile()) normalizedGivenInjection[chunkName].push('./' + _path2["default"].relative(referencePath, _path2["default"].resolve(resolvedPath, file.path)));
                  }
                } catch (err) {
                  _iterator9.e(err);
                } finally {
                  _iterator9.f();
                }
              } else if (moduleID.startsWith('./') && !moduleID.startsWith("./".concat(_path2["default"].relative(context, referencePath)))) normalizedGivenInjection[chunkName][index] = "./".concat(_path2["default"].relative(context, resolvedPath));

              index += 1;
            }
          } catch (err) {
            _iterator8.e(err);
          } finally {
            _iterator8.f();
          }
        }
      }

      return normalizedGivenInjection;
    }
    /**
     * Every injection definition type can be represented as plain object
     * (mapping from chunk name to array of module ids). This method converts
     * each representation into the normalized plain object notation.
     * @param givenInjection - Given entry injection to normalize.
     * @returns Normalized representation of given entry injection.
     */

  }, {
    key: "normalizeGivenInjection",
    value: function normalizeGivenInjection(givenInjection) {
      var result = {};
      if (Array.isArray(givenInjection)) result = {
        index: givenInjection
      };else if (typeof givenInjection === 'string') result = {
        index: [givenInjection]
      };else if (_clientnode["default"].isPlainObject(givenInjection)) {
        var hasContent = false;
        var chunkNamesToDelete = [];

        for (var chunkName in givenInjection) {
          if (Object.prototype.hasOwnProperty.call(givenInjection, chunkName)) if (Array.isArray(givenInjection[chunkName])) {
            if (givenInjection[chunkName].length > 0) {
              hasContent = true;
              result[chunkName] = givenInjection[chunkName];
            } else chunkNamesToDelete.push(chunkName);
          } else {
            hasContent = true;
            result[chunkName] = [givenInjection[chunkName]];
          }
        }

        if (hasContent) {
          var _iterator10 = _createForOfIteratorHelper(chunkNamesToDelete),
              _step10;

          try {
            for (_iterator10.s(); !(_step10 = _iterator10.n()).done;) {
              var _chunkName = _step10.value;
              delete result[_chunkName];
            }
          } catch (err) {
            _iterator10.e(err);
          } finally {
            _iterator10.f();
          }
        } else result = {
          index: []
        };
      }
      return result;
    }
    /**
     * Determines all concrete file paths for given injection which are marked
     * with the "__auto__" indicator.
     * @param givenInjection - Given entry and external injection to take
     * into account.
     * @param buildConfigurations - Resolved build configuration.
     * @param aliases - Mapping of aliases to take into account.
     * @param moduleReplacements - Mapping of replacements to take into
     * account.
     * @param extensions - List of file and module extensions to take into
     * account.
     * @param context - File path to use as starting point.
     * @param referencePath - Reference path from where local files should be
     * resolved.
     * @param pathsToIgnore - Paths which marks location to ignore.
     * @returns Given injection with resolved marked indicators.
     */

  }, {
    key: "resolveAutoInjection",
    value: function resolveAutoInjection(givenInjection, buildConfigurations) {
      var aliases = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var moduleReplacements = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var extensions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {
        file: {
          external: ['compiled.js', '.js', '.json'],
          internal: KNOWN_FILE_EXTENSIONS.map(function (suffix) {
            return ".".concat(suffix);
          })
        }
      };
      var context = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : './';
      var referencePath = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : '';
      var pathsToIgnore = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : ['.git'];

      var injection = _clientnode["default"].extend(true, {}, givenInjection);

      var moduleFilePathsToExclude = Helper.determineModuleLocations(givenInjection.autoExclude, aliases, moduleReplacements, {
        file: extensions.file.internal
      }, context, referencePath, pathsToIgnore).filePaths;

      for (var _i3 = 0, _arr3 = ['entry', 'external']; _i3 < _arr3.length; _i3++) {
        var name = _arr3[_i3];
        var injectionType = injection[name];
        /* eslint-disable curly */

        if (_clientnode["default"].isPlainObject(injectionType)) {
          for (var _i4 = 0, _Object$entries = Object.entries(injectionType); _i4 < _Object$entries.length; _i4++) {
            var _Object$entries$_i = (0, _slicedToArray2["default"])(_Object$entries[_i4], 2),
                chunkName = _Object$entries$_i[0],
                chunk = _Object$entries$_i[1];

            if (chunk === '__auto__') {
              chunk = injectionType[chunkName] = [];
              var modules = Helper.getAutoInjection(buildConfigurations, moduleFilePathsToExclude, referencePath);

              for (var subChunkName in modules) {
                if (Object.prototype.hasOwnProperty.call(modules, subChunkName)) chunk.push(modules[subChunkName]);
              }
              /*
                  Reverse array to let javaScript and main files be
                  the last ones to export them rather.
              */


              chunk.reverse();
            }
          }
        } else if (injectionType === '__auto__')
          /* eslint-enable curly */
          injection[name] = Helper.getAutoInjection(buildConfigurations, moduleFilePathsToExclude, context);
      }

      return injection;
    }
    /**
     * Determines all module file paths.
     * @param buildConfigurations - Resolved build configuration.
     * @param moduleFilePathsToExclude - A list of modules file paths to
     * exclude (specified by path or id) or a mapping from chunk names to
     * module ids.
     * @param context - File path to use as starting point.
     * @returns All determined module file paths.
     */

  }, {
    key: "getAutoInjection",
    value: function getAutoInjection(buildConfigurations, moduleFilePathsToExclude, context) {
      var result = {};
      var injectedModuleIDs = {};

      var _iterator11 = _createForOfIteratorHelper(buildConfigurations),
          _step11;

      try {
        for (_iterator11.s(); !(_step11 = _iterator11.n()).done;) {
          var buildConfiguration = _step11.value;
          if (!injectedModuleIDs[buildConfiguration.outputExtension]) injectedModuleIDs[buildConfiguration.outputExtension] = [];

          var _iterator12 = _createForOfIteratorHelper(buildConfiguration.filePaths),
              _step12;

          try {
            for (_iterator12.s(); !(_step12 = _iterator12.n()).done;) {
              var moduleFilePath = _step12.value;

              if (!moduleFilePathsToExclude.includes(moduleFilePath)) {
                var relativeModuleFilePath = "./".concat(_path2["default"].relative(context, moduleFilePath));

                var directoryPath = _path2["default"].dirname(relativeModuleFilePath);

                var baseName = _path2["default"].basename(relativeModuleFilePath, ".".concat(buildConfiguration.extension));

                var moduleID = baseName;
                if (directoryPath !== '.') moduleID = _path2["default"].join(directoryPath, baseName);
                /*
                    Ensure that each output type has only one source
                    representation.
                */

                if (!injectedModuleIDs[buildConfiguration.outputExtension].includes(moduleID)) {
                  /*
                      Ensure that same module ids and different output
                      types can be distinguished by their extension
                      (JavaScript-Modules remains without extension since
                      they will be handled first because the build
                      configurations are expected to be sorted in this
                      context).
                  */
                  if (Object.prototype.hasOwnProperty.call(result, moduleID)) result[relativeModuleFilePath] = relativeModuleFilePath;else result[moduleID] = relativeModuleFilePath;
                  injectedModuleIDs[buildConfiguration.outputExtension].push(moduleID);
                }
              }
            }
          } catch (err) {
            _iterator12.e(err);
          } finally {
            _iterator12.f();
          }
        }
      } catch (err) {
        _iterator11.e(err);
      } finally {
        _iterator11.f();
      }

      return result;
    } // TODO test

    /**
     * Determines a resolved module file path in given package path.
     * @param packagePath - Path to package to resolve in.
     * @param packageMainPropertyNames - List of package file main property
     * names to search for package representing entry module definitions.
     * @param packageAliasPropertyNames - List of package file alias property
     * names to search for package specific module aliases.
     * @param encoding - Encoding to use for file names during file traversing.
     * @returns Path if found and / or additional package aliases to consider.
     */

  }, {
    key: "determineModuleFilePathInPackage",
    value: function determineModuleFilePathInPackage(packagePath) {
      var packageMainPropertyNames = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ['main'];
      var packageAliasPropertyNames = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
      var encoding = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'utf-8';
      var result = {
        fileName: null,
        packageAliases: null
      };

      if (_clientnode["default"].isDirectorySync(packagePath)) {
        var pathToPackageJSON = _path2["default"].resolve(packagePath, 'package.json');

        if (_clientnode["default"].isFileSync(pathToPackageJSON)) {
          var localConfiguration = {};

          try {
            localConfiguration = JSON.parse(_fs["default"].readFileSync(pathToPackageJSON, {
              encoding: encoding
            }));
          } catch (error) {
            console.warn("Package configuration file \"".concat(pathToPackageJSON, "\" ") + "could not parsed: ".concat(_clientnode["default"].represent(error)));
          }

          var _iterator13 = _createForOfIteratorHelper(packageMainPropertyNames),
              _step13;

          try {
            for (_iterator13.s(); !(_step13 = _iterator13.n()).done;) {
              var propertyName = _step13.value;

              if (Object.prototype.hasOwnProperty.call(localConfiguration, propertyName) && typeof localConfiguration[propertyName] === 'string' && localConfiguration[propertyName]) {
                result.fileName = localConfiguration[propertyName];
                break;
              }
            }
          } catch (err) {
            _iterator13.e(err);
          } finally {
            _iterator13.f();
          }

          var _iterator14 = _createForOfIteratorHelper(packageAliasPropertyNames),
              _step14;

          try {
            for (_iterator14.s(); !(_step14 = _iterator14.n()).done;) {
              var _propertyName = _step14.value;

              if (Object.prototype.hasOwnProperty.call(localConfiguration, _propertyName) && _clientnode["default"].isPlainObject(localConfiguration[_propertyName])) {
                result.packageAliases = localConfiguration[_propertyName];
                break;
              }
            }
          } catch (err) {
            _iterator14.e(err);
          } finally {
            _iterator14.f();
          }
        }
      }

      return result;
    }
    /**
     * Determines a concrete file path for given module id.
     * @param moduleID - Module id to determine.
     * @param aliases - Mapping of aliases to take into account.
     * @param moduleReplacements - Mapping of replacements to take into
     * account.
     * @param extensions - List of file and module extensions to take into
     * account.
     * @param context - File path to determine relative to.
     * @param referencePath - Path to resolve local modules relative to.
     * @param pathsToIgnore - Paths which marks location to ignore.
     * @param relativeModuleLocations - List of relative file path to search
     * for modules in.
     * @param packageEntryFileNames - List of package entry file names to
     * search for. The magic name "__package__" will search for an appreciate
     * entry in a "package.json" file.
     * @param packageMainPropertyNames - List of package file main property
     * names to search for package representing entry module definitions.
     * @param packageAliasPropertyNames - List of package file alias property
     * names to search for package specific module aliases.
     * @param encoding - Encoding to use for file names during file traversing.
     * @returns File path or given module id if determinations has failed or
     * wasn't necessary.
     */

  }, {
    key: "determineModuleFilePath",
    value: function determineModuleFilePath(moduleID) {
      var aliases = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var moduleReplacements = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var extensions = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
        file: KNOWN_FILE_EXTENSIONS.map(function (suffix) {
          return ".".concat(suffix);
        })
      };
      var context = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : './';
      var referencePath = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : '';
      var pathsToIgnore = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : ['.git'];
      var relativeModuleLocations = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : ['node_modules'];
      var packageEntryFileNames = arguments.length > 8 && arguments[8] !== undefined ? arguments[8] : ['index'];
      var packageMainPropertyNames = arguments.length > 9 && arguments[9] !== undefined ? arguments[9] : ['main'];
      var packageAliasPropertyNames = arguments.length > 10 && arguments[10] !== undefined ? arguments[10] : [];
      var encoding = arguments.length > 11 && arguments[11] !== undefined ? arguments[11] : 'utf-8';
      moduleID = Helper.applyModuleReplacements(Helper.applyAliases(Helper.stripLoader(moduleID), aliases), moduleReplacements);
      if (!moduleID) return null;
      var moduleFilePath = moduleID;
      if (moduleFilePath.startsWith('./')) moduleFilePath = _path2["default"].join(referencePath, moduleFilePath);
      var moduleLocations = [referencePath].concat(relativeModuleLocations.map(function (filePath) {
        return _path2["default"].resolve(context, filePath);
      }));
      var parts = context.split('/');
      parts.splice(-1, 1);

      while (parts.length > 0) {
        var _iterator15 = _createForOfIteratorHelper(relativeModuleLocations),
            _step15;

        try {
          for (_iterator15.s(); !(_step15 = _iterator15.n()).done;) {
            var relativePath = _step15.value;
            moduleLocations.push(_path2["default"].join('/', parts.join('/'), relativePath));
          }
        } catch (err) {
          _iterator15.e(err);
        } finally {
          _iterator15.f();
        }

        parts.splice(-1, 1);
      }

      var _iterator16 = _createForOfIteratorHelper([referencePath].concat(moduleLocations)),
          _step16;

      try {
        for (_iterator16.s(); !(_step16 = _iterator16.n()).done;) {
          var moduleLocation = _step16.value;

          var _iterator17 = _createForOfIteratorHelper(['', '__package__'].concat(packageEntryFileNames)),
              _step17;

          try {
            for (_iterator17.s(); !(_step17 = _iterator17.n()).done;) {
              var fileName = _step17.value;

              var _iterator18 = _createForOfIteratorHelper([''].concat(extensions.file)),
                  _step18;

              try {
                for (_iterator18.s(); !(_step18 = _iterator18.n()).done;) {
                  var fileExtension = _step18.value;
                  var currentModuleFilePath = void 0;
                  if (moduleFilePath.startsWith('/')) currentModuleFilePath = _path2["default"].resolve(moduleFilePath);else currentModuleFilePath = _path2["default"].resolve(moduleLocation, moduleFilePath);
                  var packageAliases = {};

                  if (fileName === '__package__') {
                    var result = Helper.determineModuleFilePathInPackage(currentModuleFilePath, packageMainPropertyNames, packageAliasPropertyNames, encoding);
                    if (result.fileName) fileName = result.fileName;
                    if (result.packageAliases) packageAliases = result.packageAliases;
                    if (fileName === '__package__') continue;
                  }

                  fileName = Helper.applyModuleReplacements(Helper.applyAliases(fileName, packageAliases), moduleReplacements);
                  if (fileName) currentModuleFilePath = _path2["default"].resolve(currentModuleFilePath, "".concat(fileName).concat(fileExtension));else currentModuleFilePath += "".concat(fileName).concat(fileExtension);
                  if (Helper.isFilePathInLocation(currentModuleFilePath, pathsToIgnore)) continue;
                  if (_clientnode["default"].isFileSync(currentModuleFilePath)) return currentModuleFilePath;
                }
              } catch (err) {
                _iterator18.e(err);
              } finally {
                _iterator18.f();
              }
            }
          } catch (err) {
            _iterator17.e(err);
          } finally {
            _iterator17.f();
          }
        }
      } catch (err) {
        _iterator16.e(err);
      } finally {
        _iterator16.f();
      }

      return null;
    } // endregion

    /**
     * Determines a concrete file path for given module id.
     * @param moduleID - Module id to determine.
     * @param aliases - Mapping of aliases to take into account.
     * @returns The alias applied given module id.
     */

  }, {
    key: "applyAliases",
    value: function applyAliases(moduleID, aliases) {
      for (var alias in aliases) {
        if (alias.endsWith('$')) {
          if (moduleID === alias.substring(0, alias.length - 1)) moduleID = aliases[alias];
        } else moduleID = moduleID.replace(alias, aliases[alias]);
      }

      return moduleID;
    }
    /**
     * Determines a concrete file path for given module id.
     * @param moduleID - Module id to determine.
     * @param replacements - Mapping of regular expressions to their
     * corresponding replacements.
     * @returns The replacement applied given module id.
     */

  }, {
    key: "applyModuleReplacements",
    value: function applyModuleReplacements(moduleID, replacements) {
      for (var replacement in replacements) {
        if (Object.prototype.hasOwnProperty.call(replacements, replacement)) moduleID = moduleID.replace( // @ts-ignore: https://github.com/microsoft/TypeScript/
        // issues/22378
        new RegExp(replacement), replacements[replacement]);
      }

      return moduleID;
    }
    /**
     * Determines the nearest package configuration file from given file path.
     * @param start - Reference location to search from.
     * @param fileName - Package configuration file name.
     * @returns Determined file path.
     */

  }, {
    key: "findPackageDescriptorFilePath",
    value: function findPackageDescriptorFilePath(start) {
      var fileName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'package.json';

      if (typeof start === 'string') {
        if (!start.endsWith(_path2["default"].sep)) start += _path2["default"].sep;
        start = start.split(_path2["default"].sep);
      }

      if (!start.length) return null;
      start.pop();

      var result = _path2["default"].resolve(start.join(_path2["default"].sep), fileName);

      try {
        if (_fs["default"].existsSync(result)) return result;
        /* eslint-disable no-empty */
      } catch (error) {}
      /* eslint-enable no-empty */


      return Helper.findPackageDescriptorFilePath(start, fileName);
    }
    /**
     * Determines the nearest package configuration from given module file
     * path.
     * @param modulePath - Module path to take as reference location (leaf in
     * tree).
     * @param fileName - Package configuration file name.
     * @returns A object containing found parsed configuration an their
     * corresponding file path.
     */

  }, {
    key: "getClosestPackageDescriptor",
    value: function getClosestPackageDescriptor(modulePath) {
      var fileName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'package.json';
      var filePath = Helper.findPackageDescriptorFilePath(modulePath, fileName);
      if (!filePath) return null;
      var configuration = eval('require')(filePath);
      /*
          If the package.json does not have a name property, try again from
          one level higher.
      */

      if (!configuration.name) return Helper.getClosestPackageDescriptor(_path2["default"].resolve(_path2["default"].dirname(filePath), '..'), fileName);
      if (!configuration.version) configuration.version = 'not set';
      return {
        configuration: configuration,
        filePath: filePath
      };
    }
  }]);
  return Helper;
}();

exports.Helper = Helper;
var _default = Helper; // endregion
// region vim modline
// vim: set tabstop=4 shiftwidth=4 expandtab:
// vim: foldmethod=marker foldmarker=region,endregion:
// endregion

exports["default"] = _default;
